@@ -1,23 +1,34 @@
 --- a/setup.py
 +++ b/setup.py
-@@ -20,12 +20,15 @@ from distutils.command.build_ext import
- 
- from setuptools import Extension, setup
+@@ -8,6 +8,9 @@
+ # Your cheese is so fresh most people think it's a cream: Mascarpone
+ # ------------------------------
  
 +from sysconfig import get_platform
 +host_platform = get_platform()
 +
- # monkey patch import hook. Even though flake8 says it's not used, it is.
- # comment this out to disable multi threaded builds.
- import mp_compile
+ import os
+ import re
+ import struct
+@@ -38,7 +41,7 @@ TIFF_ROOT = None
+ ZLIB_ROOT = None
  
  
--if sys.platform == "win32" and sys.version_info >= (3, 8):
-+if host_platform == "win32" and sys.version_info >= (3, 8):
+-if sys.platform == "win32" and sys.version_info >= (3, 9):
++if host_platform == "win32" and sys.version_info >= (3, 9):
      warnings.warn(
-         "Pillow does not yet support Python {}.{} and does not yet provide "
-         "prebuilt Windows binaries. We do not recommend building from "
-@@ -131,7 +134,7 @@ def _dbg(s, tp=None):
+         "Pillow {} does not support Python {}.{} and does not provide prebuilt "
+         "Windows binaries. We do not recommend building from source on Windows.".format(
+@@ -134,7 +137,7 @@ class RequiredDependencyException(Except
+ PLATFORM_MINGW = "mingw" in ccompiler.get_default_compiler()
+ PLATFORM_PYPY = hasattr(sys, "pypy_version_info")
+ 
+-if sys.platform == "win32" and PLATFORM_MINGW:
++if host_platform == "win32" and PLATFORM_MINGW:
+     from distutils import cygwinccompiler
+ 
+     cygwin_versions = cygwinccompiler.get_versions()
+@@ -158,7 +161,7 @@ def _dbg(s, tp=None):
  def _find_library_dirs_ldconfig():
      # Based on ctypes.util from Python 2
  
@@ -26,7 +37,7 @@
          if struct.calcsize("l") == 4:
              machine = os.uname()[4] + "-32"
          else:
-@@ -153,7 +156,7 @@ def _find_library_dirs_ldconfig():
+@@ -180,7 +183,7 @@ def _find_library_dirs_ldconfig():
          env["LC_ALL"] = "C"
          env["LANG"] = "C"
  
@@ -35,7 +46,7 @@
          args = ["/sbin/ldconfig", "-r"]
          expr = r".* => (.*)"
          env = {}
-@@ -340,6 +343,38 @@ class pil_build_ext(build_ext):
+@@ -352,6 +355,38 @@ class pil_build_ext(build_ext):
                  _dbg("Requiring %s", x)
                  self.feature.required.add(x)
  
@@ -74,7 +85,7 @@
      def build_extensions(self):
  
          library_dirs = []
-@@ -417,14 +452,14 @@ class pil_build_ext(build_ext):
+@@ -429,7 +464,7 @@ class pil_build_ext(build_ext):
          if self.disable_platform_guessing:
              pass
  
@@ -83,7 +94,8 @@
              # pythonX.Y.dll.a is in the /usr/lib/pythonX.Y/config directory
              _add_directory(
                  library_dirs,
-                 os.path.join("/usr/lib", "python%s" % sys.version[:3], "config"),
+@@ -438,7 +473,7 @@ class pil_build_ext(build_ext):
+                 ),
              )
  
 -        elif sys.platform == "darwin":
@@ -91,7 +103,7 @@
              # attempt to make sure we pick freetype2 over other versions
              _add_directory(include_dirs, "/sw/include/freetype2")
              _add_directory(include_dirs, "/sw/lib/freetype2/include")
-@@ -465,13 +500,13 @@ class pil_build_ext(build_ext):
+@@ -479,13 +514,13 @@ class pil_build_ext(build_ext):
                  _add_directory(include_dirs, "/usr/X11/include")
  
          elif (
@@ -109,7 +121,7 @@
                  "ANDROID_ROOT", None
              ):
                  # termux support for android.
-@@ -482,11 +517,11 @@ class pil_build_ext(build_ext):
+@@ -496,11 +531,11 @@ class pil_build_ext(build_ext):
                      library_dirs, os.path.join(os.environ["ANDROID_ROOT"], "lib")
                  )
  
@@ -123,7 +135,7 @@
              _add_directory(library_dirs, "/opt/local/lib")
              _add_directory(include_dirs, "/opt/local/include")
  
-@@ -502,7 +537,7 @@ class pil_build_ext(build_ext):
+@@ -516,7 +551,7 @@ class pil_build_ext(build_ext):
              # alpine, at least
              _add_directory(library_dirs, "/lib")
  
@@ -132,7 +144,7 @@
              if PLATFORM_MINGW:
                  _add_directory(
                      include_dirs, "C:\\msys64\\mingw32\\include\\libimagequant"
-@@ -542,7 +577,7 @@ class pil_build_ext(build_ext):
+@@ -556,7 +591,7 @@ class pil_build_ext(build_ext):
              if _find_include_file(self, "zlib.h"):
                  if _find_library_file(self, "z"):
                      feature.zlib = "z"
@@ -141,7 +153,7 @@
                      feature.zlib = "zlib"  # alternative name
  
          if feature.want("jpeg"):
-@@ -550,7 +585,7 @@ class pil_build_ext(build_ext):
+@@ -564,7 +599,7 @@ class pil_build_ext(build_ext):
              if _find_include_file(self, "jpeglib.h"):
                  if _find_library_file(self, "jpeg"):
                      feature.jpeg = "jpeg"
@@ -150,7 +162,7 @@
                      feature.jpeg = "libjpeg"  # alternative name
  
          feature.openjpeg_version = None
-@@ -606,7 +641,7 @@ class pil_build_ext(build_ext):
+@@ -620,7 +655,7 @@ class pil_build_ext(build_ext):
              if _find_include_file(self, "tiff.h"):
                  if _find_library_file(self, "tiff"):
                      feature.tiff = "tiff"
@@ -159,31 +171,32 @@
                      self, "libtiff"
                  ):
                      feature.tiff = "libtiff"
-@@ -691,7 +726,7 @@ class pil_build_ext(build_ext):
+@@ -711,7 +746,7 @@ class pil_build_ext(build_ext):
          if feature.jpeg2000:
              libs.append(feature.jpeg2000)
              defs.append(("HAVE_OPENJPEG", None))
--            if sys.platform == "win32":
-+            if host_platform == "win32":
+-            if sys.platform == "win32" and not PLATFORM_MINGW:
++            if host_platform == "win32" and not PLATFORM_MINGW:
                  defs.append(("OPJ_STATIC", None))
          if feature.zlib:
              libs.append(feature.zlib)
-@@ -702,12 +737,12 @@ class pil_build_ext(build_ext):
-         if feature.tiff:
-             libs.append(feature.tiff)
-             defs.append(("HAVE_LIBTIFF", None))
+@@ -725,13 +760,13 @@ class pil_build_ext(build_ext):
+         if feature.xcb:
+             libs.append(feature.xcb)
+             defs.append(("HAVE_XCB", None))
 -        if sys.platform == "win32":
 +        if host_platform == "win32":
              libs.extend(["kernel32", "user32", "gdi32"])
-         if struct.unpack("h", "\0\1".encode("ascii"))[0] == 1:
+         if struct.unpack("h", b"\0\1")[0] == 1:
              defs.append(("WORDS_BIGENDIAN", None))
  
--        if sys.platform == "win32" and not (PLATFORM_PYPY or PLATFORM_MINGW):
-+        if host_platform == "win32" and not (PLATFORM_PYPY or PLATFORM_MINGW):
-             defs.append(("PILLOW_VERSION", '"\\"%s\\""' % PILLOW_VERSION))
-         else:
-             defs.append(("PILLOW_VERSION", '"%s"' % PILLOW_VERSION))
-@@ -731,7 +766,7 @@ class pil_build_ext(build_ext):
+         if (
+-            sys.platform == "win32"
++            host_platform == "win32"
+             and sys.version_info < (3, 9)
+             and not (PLATFORM_PYPY or PLATFORM_MINGW)
+         ):
+@@ -758,7 +793,7 @@ class pil_build_ext(build_ext):
  
          if feature.lcms:
              extra = []
@@ -192,7 +205,7 @@
                  extra.extend(["user32", "gdi32"])
              exts.append(
                  Extension(
-@@ -756,7 +791,7 @@ class pil_build_ext(build_ext):
+@@ -783,7 +818,7 @@ class pil_build_ext(build_ext):
                  )
              )
  
@@ -201,12 +214,12 @@
          exts.append(
              Extension(
                  "PIL._imagingtk",
-@@ -785,7 +820,7 @@ class pil_build_ext(build_ext):
+@@ -812,7 +847,7 @@ class pil_build_ext(build_ext):
          print("-" * 68)
          print("version      Pillow %s" % PILLOW_VERSION)
          v = sys.version.split("[")
--        print("platform     %s %s" % (sys.platform, v[0].strip()))
-+        print("platform     %s %s" % (host_platform, v[0].strip()))
+-        print("platform     {} {}".format(sys.platform, v[0].strip()))
++        print("platform     {} {}".format(host_platform, v[0].strip()))
          for v in v[1:]:
              print("             [%s" % v.strip())
          print("-" * 68)
